#include "mcc_generated_files/mcc.h"
#include "esp8266.h"
#include <string.h>
#include <stdio.h>

struct Buffer buffer;

// Stellt 1 Byte in den Ringbuffer
//
// Returns:
//     BUFFER_FAIL       der Ringbuffer ist voll. Es kann kein weiteres Byte gespeichert werden
//     BUFFER_SUCCESS    das Byte wurde gespeichert
//
uint8_t BufferIn(uint8_t byte)
{
  uint8_t next = ((buffer.write + 1) & BUFFER_MASK);

  if (buffer.read == next)
    return BUFFER_FAIL; // voll

  buffer.data[buffer.write] = byte;
  // buffer.data[buffer.write & BUFFER_MASK] = byte; // absolut Sicher
  buffer.write = next;
  buffer.count++;
  return BUFFER_SUCCESS;
}

//
// Holt 1 Byte aus dem Ringbuffer, sofern mindestens eines abholbereit ist
//
// Returns:
//     BUFFER_FAIL       der Ringbuffer ist leer. Es kann kein Byte geliefert werden.
//     BUFFER_SUCCESS    1 Byte wurde geliefert
//
uint8_t BufferOut(uint8_t *pByte)
{
  if (buffer.read == buffer.write)
    return BUFFER_FAIL;

  *pByte = buffer.data[buffer.read];
  buffer.count--;
  buffer.read = (buffer.read+1) & BUFFER_MASK;

  return BUFFER_SUCCESS;
}

void Uart_Interrupt(void)
{
   BufferIn(U1RXB);
}

void init_esp8266(void)
{
    buffer.read=0;
    buffer.write=0;
    UART1_SetRxInterruptHandler(Uart_Interrupt);
}

inline char BufferAvailable(void)
{
	return(buffer.write!=buffer.read);
}

inline void BufferClr(void)
{
	PIE3bits.U1RXIE = 0;
	buffer.read=buffer.write;
	PIE3bits.U1RXIE = 1;
}

char ReadcBuffer(void)
{
    char zeichen;
  if (buffer.read == buffer.write)
    return;

  zeichen = buffer.data[buffer.read];
  buffer.count--;
  buffer.read = (buffer.read+1) & BUFFER_MASK;    
  return zeichen;
}

char ReadLnBuffer(char* data)
{
	int timeoutcnt=0;
	data--;
	do
	{
		if(BufferAvailable())
		{
			*(++data)=buffer.data[buffer.read];
            buffer.count--;
            buffer.read = (buffer.read+1) & BUFFER_MASK;
			*(data+1)=0x00;
			timeoutcnt=0;
		}
		else
		{
			if((timeoutcnt++)>100)		// Timeout ~ 1500ms
				return 0;
			__delay_ms(1);
		}
	}while(*data!='\n');
	return 1;
}

char ReadBuffer(char* data, uint16_t len)
{
	int timeoutcnt=0;
    uint16_t count=0;
	data--;
	do
	{
		if(BufferAvailable())
		{
			*(++data)=buffer.data[buffer.read];
            buffer.count--;
            buffer.read = (buffer.read+1) & BUFFER_MASK;
			*(data+1)=0x00;
            count++;
			timeoutcnt=0;
		}
		else
		{
			if((timeoutcnt++)>100)		// Timeout ~ 1500ms
				return 0;
			__delay_ms(1);
		}
	}while(count!=len);
	return 1;
}

char SearchBuffer(const char* search)
{
	unsigned int timeoutcnt=0, i=0;
    char zeichen;
	unsigned int serlen;
	serlen=strlen(search);
	do
	{
		if(BufferAvailable())
		{
            zeichen=buffer.data[buffer.read];
            buffer.count--;
            buffer.read = (buffer.read+1) & BUFFER_MASK;
			if(search[i]==zeichen)
				i++;
			else
				i=0;

			if(i==serlen)
				return 1;

			timeoutcnt=0;
		}
		else
		{
			if((timeoutcnt++)>100)		// Timeout ~ 1500ms
				return 0;
			__delay_ms(1);
		}
	}while(1);
}

char SearchBuffer2(const char* search)
{
	unsigned int timeoutcnt=0, i=0;
    char zeichen;
	unsigned int serlen;
	serlen=strlen(search);
	do
	{
		if(BufferAvailable())
		{
            zeichen=buffer.data[buffer.read];
            buffer.count--;
            buffer.read = (buffer.read+1) & BUFFER_MASK;
			if(search[i]==zeichen)
				i++;
			else
				i=0;

			if(i==serlen)
				return 1;

			timeoutcnt=0;
		}
		else
		{
			if((timeoutcnt++)>1500)		// Timeout ~ 1500ms
				return 0;
			__delay_ms(1);
		}
	}while(1);
}

void PutsUART(const char* str)
{
	while(*str)
	{
		UART1_Write(*(str++));
	}
}
void PutssUART(char* str)
{
	while(*str)
	{
		UART1_Write(*(str++));
	}
}
void PutcUART(char str)
{
	UART1_Write(str);
}